@@ -7,10 +7,7 @@ Created by Maximillian Dornseif on 2010-01-16.
 Copyright (c) 2010 HUDORA. All rights reserved.
 """
 
-from __future__ import print_function
-from __future__ import unicode_literals
 from __future__ import absolute_import
 from __future__ import division
-from future_builtins import map, filter, ascii, hex, oct
 
 from pygeodb.pygeodb_core import *
