@@ -27,15 +27,12 @@ locations = GeoLocation.select()
 distanz_liste = sort(locations, referenz)
 """
 
-from __future__ import print_function
-from __future__ import unicode_literals
 from __future__ import absolute_import
-from future_builtins import map, filter, ascii, hex, oct
 
 import math
-import operator
 from pygeodb.plzdata import geodata
 
+
 class PLZ:
     def __init__(self, plz, land='DE'):
         """The python object for a geolocation"""
@@ -49,9 +46,9 @@ class PLZ:
         """Calculates the distance between two geolocations"""
         fLat, fLon = math.radians(self.latitude), math.radians(self.longitude)
         tLat, tLon = math.radians(other.latitude), math.radians(other.longitude)
-        distance = math.acos(math.sin(tLat) * math.sin(fLat)
-                             + math.cos(tLat) * math.cos(fLat) * math.cos(tLon
-                             - fLon)) * 6380000
+        distance = math.acos(
+            math.sin(tLat) * math.sin(fLat)
+            + math.cos(tLat) * math.cos(fLat) * math.cos(tLon - fLon)) * 6380000
         return int(distance)
 
 
@@ -63,10 +60,11 @@ def _obj2plz(obj):
     else:
         return PLZ(obj)
 
+
 def distance(locationa, locationb):
     plza = _obj2plz(locationa)
     plzb = _obj2plz(locationb)
-    return plza-plzb
+    return plza - plzb
 
 
 def distances(reference, locations):
